/*

    umesh2dgui, a GUI for umesh2d grid generator

    Copyright (C) 2010 Gabriele Dini Ciacci

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   Created on March 27, 2010, 9:21 PM
 */

#include "u2d_PlotmtvOutput.h"

#include "boost/numeric/mtl/mtl.hpp"

u2d_PlotmtvOutput::u2d_PlotmtvOutput() {
}

u2d_PlotmtvOutput::u2d_PlotmtvOutput(string _filename, dense2D<double> _nodes,
        dense2D<int> _elements, dense_vector<double> _values) :
filename(_filename),
nodes(_nodes),
elements(_elements),
values(_values) {
}


u2d_PlotmtvOutput::u2d_PlotmtvOutput(const u2d_PlotmtvOutput& orig) {
}

u2d_PlotmtvOutput::~u2d_PlotmtvOutput() {
}


void u2d_PlotmtvOutput::output()
    throw (u2d_IO_Exception&) {
    try {
        openFile();
        printHeader();
        for(int i = 0; i < num_cols(elements); i++) {
            int node1_id = elements(0, i);
            int node2_id = elements(1, i);
            int node3_id = elements(2, i);
            int node1_idx = elements(0, i) - 1;
            int node2_idx = elements(1, i) - 1;
            int node3_idx = elements(2, i) - 1;
            print2DNodeLine(nodes(0, node1_idx), nodes(1, node1_idx), values(node1_idx), node1_id);
            print2DNodeLine(nodes(0, node2_idx), nodes(1, node2_idx), values(node2_idx), node2_id);
            print2DNodeLine(nodes(0, node3_idx), nodes(1, node3_idx), values(node3_idx), node3_id);
            printEmptyLine();
        }
        closeFile();
    }
    catch (...) {
        throw;
    }
}

void u2d_PlotmtvOutput::setFilename(string _filename)
    throw (u2d_Parse_Error_Exception &) {
    if(_filename.empty() == true) {
        throw new u2d_Parse_Error_Exception("Specified project name is empty");
    }
    filename = _filename;
}

string u2d_PlotmtvOutput::getFilename() {
    return filename;
}


void u2d_PlotmtvOutput::setNodesCoords(dense2D<double> _nodes){
    nodes = _nodes;
}

void u2d_PlotmtvOutput::setElements(dense2D<int> _elements){
    elements = _elements;
}

void u2d_PlotmtvOutput::setNodesValues(dense_vector<double> _values){
    values = _values;
}

inline void u2d_PlotmtvOutput::print2DNodeLine(double x, double y, double val, int id) {
    fprintf(file, "%e %e %e %i\n", x, y, val, id);
}

inline void u2d_PlotmtvOutput::printHeader() {
    fprintf(file, "$ DATA = CONTCURVE\n" \
                  "%% contstyle = 2\n" \
                  "%% nsteps = 50\n" \
                  "%% meshplot  = true\n\n");
}

inline void u2d_PlotmtvOutput::printEmptyLine() {
    fprintf(file, "\n");
}

void u2d_PlotmtvOutput::openFile()
    throw (u2d_IO_Exception &) {
    file = fopen(filename.c_str(), "w+");
    if(file == NULL) {
        throw new u2d_IO_Exception("Can't open plotmtv file for output\n");
    }
}

void u2d_PlotmtvOutput::closeFile()
    throw (u2d_IO_Exception &) {
    int ret = 0;
    ret = fclose(file);
    if(ret == EOF) {
        throw new u2d_IO_Exception("Can't closen plotmtv output file\n");
    }
}