/*

    umesh2dgui, a GUI for umesh2d grid generator

    Copyright (C) 2010 Gabriele Dini Ciacci

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   Created on March 27, 2010, 9:21 PM
 */

#ifndef _U2D_PLOTMTVOUTPUT_H
#define	_U2D_PLOTMTVOUTPUT_H

#include <string>
#include <exceptions.h>

#include "u2d_file_output.h"
#include "boost/numeric/mtl/mtl.hpp"

using std::string;
using namespace mtl;

class u2d_PlotmtvOutput : public u2d_FileOutput {
public:
    u2d_PlotmtvOutput();
    u2d_PlotmtvOutput(string _filename, dense2D<double> _nodes,
        dense2D<int> _elements, dense_vector<double> _values);
    u2d_PlotmtvOutput(const u2d_PlotmtvOutput& orig);
    virtual ~u2d_PlotmtvOutput();
    void output() throw (u2d_IO_Exception &);
    void setFilename(string filename) throw (u2d_Parse_Error_Exception &);
    string getFilename();
    void setNodesCoords(dense2D<double> nodes);
    void setElements(dense2D<int> indexes);
    void setNodesValues(dense_vector<double> values);
private:
    string filename;
    dense2D<double> nodes;
    dense2D<int> elements;
    dense_vector<double> values;
    void openFile() throw (u2d_IO_Exception &);
    void closeFile() throw (u2d_IO_Exception &);
    FILE *file;
    inline void printHeader();
    inline void print2DNodeLine(double x, double y, double val, int id);
    inline void printEmptyLine();
};

#endif	/* _U2D_PLOTMTVOUTPUT_H */

