/* 

    umesh2dgui, a GUI for umesh2d grid generator 

    Copyright (C) 2008 Gabriele Dini Ciacci
  
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/


#ifndef U2D_UMESH2D_OUTPUT_H
#define U2D_UMESH2D_OUTPUT_H

#include <string>

#include "u2d_core.h"
#include "u2d_file_output.h"
#include "exceptions.h"

#define FORMAT_UMESH2D 1
#define FORMAT_UMESH2D_FE 2
#define FORMAT_UMESH2D_FV 3

#define DIR_SEPARATOR '/'

using std::string;

class u2d_Umesh2dOutput : public u2d_FileOutput {
public:
    /* virtual from u2d_File_Output */
    void output() throw (u2d_IO_Exception&);
    void setFilename(string filename) throw (u2d_Parse_Error_Exception &);
    string getFilename();
    void setFormat(int format);

    /* CONSTRUCTOR */
    u2d_Umesh2dOutput();

    /* PRIMITIVE LIST SELECTION */
    void select_prim_list(Primitive *first, Primitive *last, int prim_number, int vertex_n);

    /* DOMAIN FILE SELECTION */
    void set_steiners(steiner *steiners_, int number);
    void set_backgrids(backgrid *backgrid_, int number);

private:
    /* OUTPUT TYPE SELECTION */
    int output_format;
    /* DOMAIN FILE INFORMATION */
    steiner *steiners;
    backgrid *backgrids;
    int steiner_n;
    int backgrid_n;

    /* PRIMITIVE INFORMATION */
    Primitive *firstprimitive;
    Primitive *lastprimitive;
    int elements; // number of primitives in the list, length of the list.
    int vertexs; // no idea what is this!

    /* FILE GENERATION */
    inline void printhash(FILE *output);
    inline void printformat(FILE *output, int format);
    inline void printgeom(FILE *output);
    inline void printtopo(FILE *output);
    inline void printnvne(FILE *output);
    void printverelem(FILE *output);
    inline void printish(FILE *output);
    inline void printenbvev(FILE *output);
    void printprimg(FILE *output, Primitive *curprim);
    void printprimd(FILE *outptu, Primitive *curprim);
    void printbegin(FILE *output, int id);
    void printbegin(FILE *output, int id, char *name);
    inline void printend(FILE *output);
    void printpriminfo(FILE *output);
    void printprimdata(FILE *output);

    /* FILE OUTPUT */
    void output_boundary_file() throw (u2d_IO_Exception &);
    void output_domain_file() throw (u2d_IO_Exception &);
    void open_boundary_file() throw (u2d_IO_Exception &);
    void open_domain_file() throw (u2d_IO_Exception &);
    string project_name;
    string boundary_filename; //Must allready contain rigth boundary.name
    string domain_filename; //Must allready contain rigth domain.name
    void set_boundary_filename();
    void set_domain_filename();
    FILE *boundary_file;
    FILE *domain_file;


    /* Scheduled for removal */
    void add_boundary(char *buffer);
    /* Scheduled for removal */
    void add_domain(char *buffer);
};


#endif // U2D_UMESH2D_OUTPUT_H
