/* 

    umesh2dgui, a GUI for umesh2d grid generator 

    Copyright (C) 2006 Gabriele Dini Ciacci
    Copyright (C) 2007 Gabriele Dini Ciacci

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/
#ifndef U2D_LOAD_PLOTMTV_H
#define U2D_LOAD_PLOTMTV_H

#include <u2d_GLmesh.h>

#define KNOWN_FOMARTS 27
#define UNDEFINED -1

/* Code from plotmtv source, NOT GPL.
 * unknown license, probably public domain
 */
#define CURVE2D  1
#define CURVE3D  2
#define CONTCURV 3
#define RECTGRID 4
#define GRID4D   5
#define MESH4D   6
#define POLYGON  7
#define VECTOR   8
#define MESH     9
#define COLUMN   10
#define PROBAB   11
#define HISTOG   12
#define BARCHART 13
#define UNRECOG  14

typedef struct {
  const char *label;
  int   value;
} MPdata_format;

static MPdata_format format_pairs[] = {
   {"curve2d"      , CURVE2D },
   {"2d"           , CURVE2D },
   {"twod"         , CURVE2D },
   {"curve3d"      , CURVE3D },
   {"3d"           , CURVE3D },
   {"threed"       , CURVE3D },
   {"contcurv"     , CONTCURV},
   {"contcurve"    , CONTCURV},
   {"contourcurve" , CONTCURV},
   {"contour-curve", CONTCURV},
   {"contour"      , RECTGRID},
   {"rectgrid"     , RECTGRID},
   {"grid4d"       , GRID4D  },
   {"4d"           , GRID4D  },
   {"poly"         , POLYGON },
   {"polygon"      , POLYGON },
   {"vector"       , VECTOR  },
   {"vect"         , VECTOR  },
   {"column"       , COLUMN  },
   {"prob"         , PROBAB  },
   {"probab"       , PROBAB  },
   {"probability"  , PROBAB  },
   {"hist"         , HISTOG  },
   {"histog"       , HISTOG  },
   {"histogram"    , HISTOG  },
   {"bar"          , BARCHART},
   {"barchart"     , BARCHART},
};
/* end of code from plotmtv */

class u2d_Load_PlotMtv {

 public:
  u2d_Load_PlotMtv(const char *filename);
  ~u2d_Load_PlotMtv();
  vector<u2d_Display_Set> display_set;

 private:
  int set_filename(const char *filename);
  void open_plotmtv_file();
  void read_plotmtv_file();
  char *skip_white_spaces(char *str, int length);
  int parse_header(char *header, int length);
  int parse_option(char *cur_pos,\
                   u2d_Display_Set *m_display_set);
  int parse_CURVE2D(char *cur_pos,\
                    u2d_Display_Set *m_display_set);
  void select_appropriate_GLtype(int data_type,\
                                 u2d_Display_Set *m_display_set);
  char *filename_expanded;
  FILE *file;
};

#endif
