/*

    umesh2dgui, a GUI for umesh2d grid generator

    Copyright (C) 2010 Gabriele Dini Ciacci <dark.schneider@iol.it>
    Based on a code from Luigi Quartapelle and Massimo Biava

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Created on January 20, 2010, 12:30 PM
 */

#ifndef U2D_DIRICHLETCONDITION_H
#define	U2D_DIRICHLETCONDITION_H

#include <iostream>
#include <set> // for std::set
#include <vector> // for std::vector

#include "boost/numeric/mtl/mtl.hpp"
#include "u2d_mesh.h"


using namespace std;

class u2d_DirichletCondition {
public:
    u2d_DirichletCondition();
    u2d_DirichletCondition(const u2d_DirichletCondition& orig);
    virtual ~u2d_DirichletCondition();
    int setSide(int side_id);
    bool isDirichlet(int side_id);
    void generateNodes(u2d_Mesh* mesh);

    dense_vector<int> dirichlet_nodes; /* Nodes where to enforce Dirichlet condition */
    set<int> dirichlet_unique_nodes; /* *Unique* nodes where to enforce Dirichlet condition */
private:
    void countNodes(u2d_Mesh *mesh);
    int dirichlet_nodes_n;
    int dirichlet_unique_nodes_n;
    std::vector<int> dir_sides;



};

#endif	/* U2D_DIRICHLETCONDITION_H */

