/* 
 * File:   u2d_IsoElement1DP12Points.h
 * Author: Gabriele Dini Ciacci <dark.schneider@iol.it>
 *
 * Created on January 17, 2010, 8:40 PM
 */

#ifndef U2D_ISOELEMENT1DP12POINTS_H
#define	U2D_ISOELEMENT1DP12POINTS_H

#include "u2d_mesh.h"
#include "boost/numeric/mtl/mtl.hpp"
#include "u2d_GaussPoints.h"

#define P1_1D_DEGREE_FREEDOM BOUNDARY_ELEMENT_SIZE

using namespace mtl;

class u2d_IsoElement1DP12Points {
public:
    u2d_IsoElement1DP12Points();
    u2d_IsoElement1DP12Points(const u2d_IsoElement1DP12Points& orig);
    virtual ~u2d_IsoElement1DP12Points();

    int gauss_points_n;
    /* we are linear, so we have same number of degree of freedom
     * as we have nodes per element */
    dense2D<double> gauss_points;
    dense2D<double> w;
    dense2D<double> dx;
    dense_vector<double> p;
    u2d_BoundaryGaussPoints *applyTo(u2d_Mesh *mesh);

private:

    double shapeP1Node1(double x);
    double shapeP1Node2(double x);

};

/* Inline functions must be in header file, hence if they ever
 * get called from another .cpp file (that is not the one where
 * you placed them. You get an "unresolved external" from the
 * linker
 */
inline double u2d_IsoElement1DP12Points::shapeP1Node1(double x) {
    return (1.0 - x) / 2.0;
}

inline double u2d_IsoElement1DP12Points::shapeP1Node2(double x) {
    return (x + 1.0) / 2.0;
}


#endif	/* U2D_ISOELEMENT1DP12POINTS_H */

