/* 
 * File:   IsoElement2DP13Points.h
 * Author: Gabriele Dini Ciacci <dark.schneider@iol.it>
 *
 * Created on January 17, 2010, 8:15 PM
 */

#ifndef U2D_ISOELEMENT2DP13POINTS_H
#define	U2D_ISOELEMENT2DP13POINTS_H

#include "u2d_mesh.h"
#include "boost/numeric/mtl/mtl.hpp"
#include "u2d_GaussPoints.h"

#define P1_2D_DEGREE_FREEDOM DOMAIN_ELEMENT_SIZE

using namespace mtl;

class u2d_IsoElement2DP13Points {
public:
    u2d_IsoElement2DP13Points();
    u2d_IsoElement2DP13Points(const u2d_IsoElement2DP13Points& orig);
    virtual ~u2d_IsoElement2DP13Points();

    int gauss_points_n;
    /* we are linear, so we have same number of degree of freedom
     * as we have nodes per element */
    dense2D<double> gauss_points;
    dense2D<double> w;
    dense2D<double> dx;
    dense2D<double> dy;
    dense_vector<double> p;
    u2d_DomainGaussPoints *applyTo(u2d_Mesh *mesh);
    
private:

    double shapeP1Node1(double x, double y);
    double shapeP1Node2(double x, double y);
    double shapeP1Node3(double x, double y);


};

/* Inline functions must be in header file, hence if they ever
 * get called from another .cpp file (that is not the one where
 * you placed them. You get an "unresolved external" from the
 * linker
 */
inline double u2d_IsoElement2DP13Points::shapeP1Node1(double x, double y) {
    return 1 - x - y;
}

inline double u2d_IsoElement2DP13Points::shapeP1Node2(double x, double y) {
    return x;
}

inline double u2d_IsoElement2DP13Points::shapeP1Node3(double x, double y) {
    return y;
}

#endif	/* U2D_ISOELEMENT2DP13POINTS_H */

