/* 

    umesh2dgui, a GUI for umesh2d grid generator 

    Copyright (C) 2003 Gabriele Dini Ciacci
    Copyright (C) 2004 Gabriele Dini Ciacci
    Copyright (C) 2005 Gabriele Dini Ciacci
    Copyright (C) 2006 Gabriele Dini Ciacci
  
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>

/* for fstat, they probably are not portable */
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include "u2d_core.h"

bool isnear(double toll, double x, double y){

  if(fabs(x - y)  > toll) return false;
  else return true;

}

bool isnear(double toll, double x1, double y1, double x2, double y2){

  if(hypot(x1-x2,y1-y2) > toll) return false;
  else return true;

}

bool isnear(double toll, Primitive *primX, Primitive *primY){
  /* for now do not use this */
  /* we must check 4 combinations of points and first I have to
   * figure out witch is better way to do this*/

  //  rvalue = false;
  //  if(isnear(toll, primX->x1, primY->y1, primX->x2, primY->y2))
  return false;

}

double triangle_area(double ref_x, double ref_y,\
		     double x1, double y1,\
		     double x2, double y2){
  double area;
  area = 0.5 * ((x1 - ref_x) * (y2 - ref_y) -\
		(x2 - ref_x) * (y1 - ref_y));
  return area;
}

Primitive::Primitive(){
  nextprimitive = NULL;
  prevprimitive = NULL;
  maxdata = 0;
  curdata = 0;
  data = NULL;
  object = 0;
  object_part = 0;
  isboundary = false;
  beginvertex = 0;
  endvertex = 0;
}


double Primitive::getdata(int x, int y) {
  double localdata;
  localdata = this->data[y+3*x];
  return localdata;
}

void Primitive::setdata(int x, int y, double value) {
  this->data[y+3*x] = value;
}

int Primitive::allocatedata(int ndata) {
  if(ndata == 0){
    /* We requested to allocate zero data */
    fprintf(stdout, "Requested an alocation of zero [i|s|h]data, skipping data allocation\n");
    return 0;
  }
  else if(this->data == NULL) {
    if(this->maxdata == 0) {
#ifdef DEBUGGING
      fprintf(stdout, "Allocating a new mem chunk for [i|s|h]data for " \
                      "%i elements (%li bytes): ", ndata, ndata*3*sizeof(double));
#endif
      this->data = (double *)calloc(sizeof(double), ndata * 3);
      if(this->data == NULL) { 
#ifdef DEBUGGING
	fprintf(stdout, "Failed\n");
#endif
	fprintf(stderr, "[i|s|h]data allocation failed\n");
	return -1;  // memory allocation failed
      }
      else {
#ifdef DEBUGGING
	fprintf(stdout, "Success\n");
	this->maxdata = ndata;
#endif
	return 0; // OK!
      }
    }
    else {
      fprintf(stderr, "WTF?! data is null but maxdata is not 0!!\n");
      return -2; // Primitive data mismatch
    }
  }
  else {
    if(ndata < (this->maxdata) * 2) { 
      /* We dont' reallocate memory if the mem avaiable is already 
       * big enougth but less than double of avaiable. We waste but 
       * not too much
       */ 
#ifdef DEBUGGING
      fprintf(stdout, "Data chunk was allready enougth big (but less" \
                      "than double), we waste mem but save cpu\n");
#endif
      return 0;
    }
    else {
#ifdef DEBUGGING
      fprintf(stdout, "Reallocating old mem chunk of %i elements (%li bytes)" \
                      "to a new chunk of %i elements (%li bytes): ", \
                       this->maxdata, this->maxdata*3*sizeof(double),\
                       ndata, ndata*3*sizeof(double));
#endif
      this->data = (double *)realloc(this->data, 3*sizeof(double)*ndata);
      if(this->data == NULL) {
#ifdef DEBUGGING
	fprintf(stdout, "Failed\n");
#endif
	fprintf(stderr, "[i|s|h]data reallocation failed\n");
	return -1;  // memory reallocation failed
      }
      else {
#ifdef DEBUGGING
	fprintf(stdout, "Success\n");
#endif
	this->maxdata = ndata;
	return 0; // OK!
      }
    }
  }
  return -3; // How the hell we got here???!!!!
}

int Line::set_coord(int spli,\
		     double X1, double Y1,\
		     double X2, double Y2){
  spline = spli;
  x1 = X1;
  y1 = Y1;
  x2 = X2;
  y2 = Y2;
  return 0;
}

double Line::calc_area(double ref_x, double ref_y){
  double area;

  if(this->beginvertex == this->endvertex){
    fprintf(stderr, "Error calculating area\n");
    return 0;
  }

  area = triangle_area(ref_x, ref_y,\
		       this->x1, this->y1,\
		       this->x2, this->y2);

  return area;
  /* I think this is wrong
  if(this->beginvertex < this->endvertex){
    return area;
  }
  else{
    return -area;
  }
  */
}

void Line::reverse(){
  double x;
  double y;
  int vert;

  /* Invert extremes */
  x = this->x1;
  y = this->y1;
  this->x1 = this->x2;
  this->y1 = this->y2;
  this->x2 = x;
  this->y2 = y;

  /* Invert vetexes */
  vert = this->beginvertex;
  this->beginvertex = this->endvertex;
  this->endvertex = vert;
}

void Circle::find_extremes(){
  x1 = centerx + radius*cos(startarc / 360 * 2.0 * PI);
  y1 = centery + radius*sin(startarc / 360 * 2.0 * PI);
  x2 = centerx + radius*cos(endarc / 360 * 2.0 * PI);
  y2 = centery + radius*sin(endarc / 360 * 2.0 * PI);
}

int Circle::set_coord(int spli, double rad,\
		       double centx, double centy,\
		       double sarc, double earc){
  this->spline = spli;
  this->radius = rad;
  this->centerx = centx;
  this->centery = centy;
  if(isnear(DEFAULT_TOLL, sarc, earc)){
    /* circle is complete */
    sarc = 0;
    earc = 360;
      //    earc = earc + 360;
  }
  this->startarc = sarc;
  this->endarc = earc;
  this->find_extremes();
  return 0;
}

double Circle::calc_area(double ref_x, double ref_y){
  double area = 0;

  if(this->startarc < this->endarc){
    /* Start and end arc are _not_ the same, start arc is _smaller_ than end arc */
    for(int i = 1; i < CIRCLE_APPROX; i++){
      if((2.0 * PI * i / CIRCLE_APPROX + this->startarc / 360.0 * PI * 2.0 <= \
	  this->endarc / 360.0 * PI * 2.0)){
	area = area + triangle_area(ref_x, ref_y,
				    this->centerx + this->radius * \
				    cos(2.0 * PI * (i - 1) / CIRCLE_APPROX + this->startarc / 360.0 * PI * 2.0), \
				    this->centery + this->radius * \
				    sin(2.0 * PI * (i - 1) / CIRCLE_APPROX + this->startarc / 360.0 * PI * 2.0),
				    this->centerx + this->radius * \
				    cos(2.0 * PI * i / CIRCLE_APPROX + this->startarc / 360.0 * PI * 2.0), \
				    this->centery + this->radius * \
				    sin(2.0 * PI * i / CIRCLE_APPROX + this->startarc / 360.0 * PI * 2.0));
      }
    }
  }
  else if(this->startarc >= this->endarc){
    /* Start and end arc are the same or start arc is _greater_ than end arc */
    for(int i = 1; i <= CIRCLE_APPROX; i++){
      if(this->startarc / 360.0 * PI * 2.0 - 2.0 * PI * i / CIRCLE_APPROX >= \
	  this->endarc / 360.0 * PI * 2.0){
	area = area + triangle_area(ref_x, ref_y,
				    this->centerx + this->radius * \
				    cos(this->startarc / 360.0 * PI * 2.0 - 2.0 * PI * (i - 1) / CIRCLE_APPROX), \
				    this->centery + this->radius * \
				    sin(this->startarc / 360.0 * PI * 2.0 - 2.0 * PI * (i - 1) / CIRCLE_APPROX),
				    this->centerx + this->radius * \
				    cos(this->startarc / 360.0 * PI * 2.0 - 2.0 * PI * i / CIRCLE_APPROX), \
				    this->centery + this->radius * \
				    sin(this->startarc / 360.0 * PI * 2.0 - 2.0 * PI * i / CIRCLE_APPROX));
      }
    }
  }

  return area;

  /* I think this is wrong
  if(this->beginvertex < this->endvertex){
    return area;
  }
  else{
    return -area;
  }
  */
}

void Circle::reverse(){
  /* Note: find_extremes() method is conflicting with this one!!
   * but if we are reversing the vertexes we should not call anymore
   * find_extremes that should be called only by set_coord().
   * Note: Conflict shoudl be fixed now, since we are inverting
   * startarc and endarc too
   */
  double x;
  double y;
  double arc;
  int vert;

  /* Invert extremes */
  x = this->x1;
  y = this->y1;
  this->x1 = this->x2;
  this->y1 = this->y2;
  this->x2 = x;
  this->y2 = y;

  /* Invert vetexes */
  vert = this->beginvertex;
  this->beginvertex = this->endvertex;
  this->endvertex = vert;

  /* Invert start and end arc */
  arc = this->endarc;
  this->endarc = this->startarc;
  this->startarc = arc;

}

void Ellipse::find_extremes(){
  x1 = centerx + semix * cos(startarc / 360 * 2.0 * PI);
  y1 = centery + semiy * sin(startarc / 360 * 2.0 * PI);
  x2 = centerx + semix * cos(endarc / 360 * 2.0 * PI);
  y2 = centery + semiy * sin(endarc / 360 * 2.0 * PI);
}

int Ellipse::set_coord(int spli,\
			double semx, double semy,\
			double centx, double centy,\
			double sarc, double earc){
  this->spline = spli;
  this->semix = semx;
  this->semiy = semy;
  this->centerx = centx;
  this->centery = centy;
  if(isnear(DEFAULT_TOLL, sarc, earc)){
    /* ellipse is complete */
    sarc = 0;
    earc = 360;
    //    earc = earc + 360;
  }
  this->startarc = sarc;
  this->endarc = earc;
  this->find_extremes();
  return 0;
}

double Ellipse::calc_area(double ref_x, double ref_y){
  double area = 0;

  if(this->startarc < this->endarc){
    /* Start and end arc are _not_ the same, start arc is _smaller_ than end arc */
    for(int i = 1; i <= ELLIPSE_APPROX; i++){
      if((2.0 * PI * i / ELLIPSE_APPROX + this->startarc / 360.0 * PI * 2.0 <= \
	  this->endarc / 360.0 * PI * 2.0)){
	area = area + triangle_area(ref_x, ref_y,
				    this->centerx + this->semix * \
				    cos(2.0 * PI * (i - 1) / ELLIPSE_APPROX + this->startarc / 360.0 * PI * 2.0), \
				    this->centery + this->semiy * \
				    sin(2.0 * PI * (i - 1)/ ELLIPSE_APPROX + this->startarc / 360.0 * PI * 2.0),
				    this->centerx + this->semix * \
				    cos(2.0 * PI * i / ELLIPSE_APPROX + this->startarc / 360.0 * PI * 2.0), \
				    this->centery + this->semiy * \
				    sin(2.0 * PI * i / ELLIPSE_APPROX + this->startarc / 360.0 * PI * 2.0));
      }
    }
  }
  else if(this->startarc >= this->endarc){
    /* Start and end arc are the same or start arc is _greater_ than end arc */
    for(int i = 1; i <= CIRCLE_APPROX; i++){
      if(this->startarc / 360.0 * PI * 2.0 - 2.0 * PI * i / CIRCLE_APPROX >= \
	  this->endarc / 360.0 * PI * 2.0){
	area = area + triangle_area(ref_x, ref_y,
				    this->centerx + this->semix * \
				    cos(this->startarc / 360.0 * PI * 2.0 - 2.0 * PI * (i - 1) / CIRCLE_APPROX), \
				    this->centery + this->semiy * \
				    sin(this->startarc / 360.0 * PI * 2.0 - 2.0 * PI * (i - 1) / CIRCLE_APPROX),
				    this->centerx + this->semix * \
				    cos(this->startarc / 360.0 * PI * 2.0 - 2.0 * PI * i / CIRCLE_APPROX), \
				    this->centery + this->semiy * \
				    sin(this->startarc / 360.0 * PI * 2.0 - 2.0 * PI * i / CIRCLE_APPROX));
      }
    }
  }

  return area;
  /* I think this is wrong
  if(this->beginvertex < this->endvertex){
    return area;
  }
  else{
    return -area;
  }
  */
}

void Ellipse::reverse(){
  /* Note: find_extremes() method is conflicting with this one!!
   * but if we are reversing the vertexes we should not call anymore
   * find_extremes that should be called only by set_coord()
   * Note: Conflict shoudl be fixed now, since we are inverting
   * startarc and endarc too
   */
  double x;
  double y;
  double arc;
  int vert;

  /* Invert extremes */
  x = this->x1;
  y = this->y1;
  this->x1 = this->x2;
  this->y1 = this->y2;
  this->x2 = x;
  this->y2 = y;

  /* Invert vetexes */
  vert = this->beginvertex;
  this->beginvertex = this->endvertex;
  this->endvertex = vert;

  /* Invert start and end arc */
  arc = this->endarc;
  this->endarc = this->startarc;
  this->startarc = arc;
}

int Data::set_coord(const char* fname,\
		     double tx, double ty,\
		     double rot) {

  //  printf("set_coord: Fname is %s\n", fname);
  if(this->allocatefilename(strlen(fname) + 1) != 0) {
    return UNKNOWN;
  }

  strncpy(this->filename, fname, strlen(fname) + 1);
#ifdef DEBUGGING
  printf("set_coord: Filename is %s\n", filename);
#endif
  this->trasx = tx;
  this->trasy = ty;
  this->rotation = rot;


  /* We aquire file points here because aquiring
   * them inside find_extremes() would make it hard
   * to return a error condition since find_extremes()
   * is a base Primitive method
   */
  int val = 0;

  /* Check is the filename is a regular file */
  struct stat status;
  val = stat(this->filename, &status);
  if(val != 0) {
    fprintf(stderr, "Data::set_coord: Error stat file \"%s\"\n", filename);
    perror("Data::set_coord:");
    return NO_EXIST;
  }

  if(S_ISREG(status.st_mode) != 1){
    fprintf(stderr, "Data::set_coord: File \"%s\" is not a regular file\n", filename);
    return NOT_A_FILE;
  }

  /* It's a file, now open it */
  FILE *file = NULL; 
  file = fopen(this->filename, "r");
  printf("file %p\n", file);
  if(file == NULL) {
    fprintf(stderr, "Data::set_coord: Error opening file \"%s\". Permission perheps?\n", filename);
    perror("Data::set_coord:");
    return NO_FILE;
  }

  /* Acquire the file */
  /* WARNING */
  /* fast file acquisition, REWRITE */
  val = fscanf(file, "%d\n", &npoints);
  printf("File should contain %i points\n", npoints);
  if(val != 1) {
    fprintf(stderr, "Data::set_coord: Cannot acquire number of "\
	    "points from file %s. Check the first line to "\
	    "contain only one decimal number and that it is the "\
	    "number of points in the list\n", filename);
    fclose(file);
    return INVALID_FILE;
  }
  points = (double *)calloc(npoints, 2*sizeof(double)); 
  for(int i = 1; i < npoints*2; i = i + 2) {
    val = fscanf(file, "%lf %lf\n", (points + i - 1), (points + i));
    if(val != 2) {
      fprintf(stderr, "Data::set_coord: Cannot acquire correctly from"\
	      "file %s. Check line %i "\
	      "is in the correct format i.e. 0.034 0.23\n",\
	      filename, i);
      fclose(file);
      return INVALID_FILE;
    }
    printf("Acquired point: (%f, %f)\n", *(points + i - 1), *(points + i));
  }

  fclose(file);
  find_extremes();
  return 0;
}

void Data::find_extremes() {

  x1 = rot_matrixX(rotation, points[0] + trasx, points[1] + trasy);
  y1 = rot_matrixY(rotation, points[0] + trasx, points[1] + trasy);
  x2 = rot_matrixX(rotation, points[npoints - 1] + trasx, points[npoints] + trasy);
  y2 = rot_matrixY(rotation, points[npoints - 1] + trasx, points[npoints] + trasy);
}

double Data::calc_area(double ref_x, double ref_y){
  double area = 0;

  for(int i = 1; i < (npoints - 1)*2; i = i + 2) {
    area = area + triangle_area(ref_x, ref_y,\
				points[i-1], points[i],\
				points[i + 1], points[i + 2]);
  }
  /* area of triangle last point and first point (in this order)*/
  area = area + triangle_area(ref_x, ref_y,\
			      points[npoints*2 - 1], points[npoints*2],\
			      points[0], points[1]);

  return area;
}

void Data::reverse() {
  fprintf(stderr, "Data::reverse: WARNING reverse of data"\
	  "is not supported\n");\
}

int Data::allocatefilename(int length){
  if(this->filename != NULL) {
    free(this->filename);
  }
  this->filename = (char *)malloc(length);
  if(filename == NULL){
    fprintf(stderr, "Data::allocatefilename: Filename allocation problem\n");
    return -1;
  }
  memset(this->filename, 0, length);
  return 0;
}

u2d_Core::u2d_Core(){
  firstprimitive = NULL;
  lastprimitive = NULL;
  globalprimid = 0;
  elements = 0;
  memset(prim_string, 0, 1024);
  vertex_n = 1;
  last_object = 0;
  steiners = NULL;
  backgrids = NULL;
  steiner_n = 0;
  backgrid_n = 0;
}

u2d_Core::~u2d_Core(){
  /* Free steiner and backgrids arrays */
  if(steiners != NULL) {
    free(steiners);
  }
  if(backgrids != NULL) {
    free(backgrids);
  }


  /* PLANCEHOLDER, DEALLOCATE THE PRIMITIVE LIST*/
}

/* PRIMITIVE LIST MANGEMENT */

Primitive *u2d_Core::new_prim(int type) {
  Primitive *prim = NULL;
  elements++;
  //  vertexs++;
  switch(type) {
  case PR_LINE:
    prim = new Line;
    break;
  case PR_CIRCLE:
    prim = new Circle;
    break;
  case PR_ELLIPSE:
    prim = new Ellipse;
    break;
  case PR_DATA:
    prim = new Data;
    break;
  default:
    elements--;
    //    vertexs--;
    fprintf(stderr, "Unknown primitive type\n");
  }
  return prim;
}

Primitive *u2d_Core::add_prim(int type) {
  Primitive *newprim;
  newprim = new_prim(type);
    if(firstprimitive == NULL){
      firstprimitive = newprim;
    }
    else {
      lastprimitive->nextprimitive = newprim;
    }
    newprim->prevprimitive = lastprimitive;
    lastprimitive = newprim;
    newprim->id = globalprimid++;
    return newprim;
}

/* Insert a primitive after the primitive position */
void u2d_Core::insert_down_prim(Primitive *position, Primitive *insertion){
  if(position == NULL){
    printf("insert_down_prim: position is NULL, can't insert after it\n");
    return;
  }
  if(position == insertion){
    return;
  }

  /* Since position is not NULL and position is not same as insertion
   * we should be the middle of the list.
   */
  insertion->prevprimitive = position;
  insertion->nextprimitive = position->nextprimitive;

  if(position->nextprimitive != NULL){
    position->nextprimitive->prevprimitive = insertion;
  }
  else {
    lastprimitive = insertion;
  }

  position->nextprimitive = insertion;
}

/* Remove a primitive from the list */
void u2d_Core::remove_prim(Primitive *oldprim){
  if(oldprim == NULL){
    /* Just to be sure none pass us a NULL pointer */
    return;
  }
  else if(oldprim == firstprimitive && oldprim->nextprimitive != NULL){
    /* We want to delete the firstprimitive and it's not the only one */
    firstprimitive = oldprim->nextprimitive;
    oldprim->nextprimitive->prevprimitive = NULL;
  }
  else if(oldprim == firstprimitive && oldprim->nextprimitive == NULL){
    /* We want to delete the firstprimitive and it's the only one */
    firstprimitive = NULL;
    lastprimitive = NULL;
  }
  else if(oldprim->nextprimitive != NULL) {
    /* oldprim is not the first, because we checked that above
     * and is not the last because it points to anoter one. It
     * is in the middle of the list */
    oldprim->prevprimitive->nextprimitive = oldprim->nextprimitive;
    oldprim->nextprimitive->prevprimitive = oldprim->prevprimitive;
  }
  else if (oldprim->nextprimitive == NULL) {
    /* oldprim should be the last, but it is not the first and only
     * because we checked that above */
    oldprim->prevprimitive->nextprimitive = NULL;
    lastprimitive = oldprim->prevprimitive;
  }
}

void u2d_Core::move_insert_down_prim(Primitive *position, Primitive *insertion){
  if(position == NULL){
    printf("move_insert_down_prim: position is NULL, can't insert after it\n");
    return;
  }

  /* We remove it from the list */
  remove_prim(insertion);

  /* We insert it in the new position */
  insert_down_prim(position, insertion);


  /* We are the only element int he list, we do notting */
  /*  if(insertion == firstprimitive && firstprimitive->nextprimitive == NULL){
   *    return;
   *  }
   */

  /* We are the list head */
  /*  if(insertion == firstprimitive){
   *    firstprimitive = firstprimitive->nextprimitive;
   *  }
   */

}

void u2d_Core::del_prim(Primitive *oldprim) {
  remove_prim(oldprim);
  delete oldprim;
  elements--;
}

int u2d_Core::get_prim_number(){
  return elements;
}

Primitive *u2d_Core::get_prim_list_start(){
  return firstprimitive;
}
Primitive *u2d_Core::get_prim_list_end(){
  return lastprimitive;
}

/* I could have impemented a get_prim_string method
 * for each Primitive class type, but this way we lose
 * speed and space since we could have many istances
 * of Primitive class).
 * I have preferred to use an old "static"
 * function placed in the core class. It is real faster
 */
char *u2d_Core::get_prim_string(Primitive *curprim) {
  memset(prim_string, 0, 1024);

  /* Changed to the more C style below
  char primname[34];
  if(strlen(curprim->primname) > 0){
    strncpy(primname, curprim->primname, 34);
  }
  else{
    strncpy(primname, "None", 34);
  }
  */
  switch(curprim->type) {
  case PR_LINE:
    snprintf(prim_string, 1024, \
	     "%i\t%s\t%i\tLine\t%0.2f\t%0.2f\t%0.2f\t%0.2f",
	     curprim->id,
	     strlen(curprim->primname) > 0 ? curprim->primname : "None",
	     curprim->object,
	     ((Line *)curprim)->x1,
	     ((Line *)curprim)->y1,
	     ((Line *)curprim)->x2,
	     ((Line *)curprim)->y2
	     );
    break;
  case PR_CIRCLE:
    snprintf(prim_string, 1024, \
	     "%i\t%s\t%i\tCircle\t%0.2f\t%0.2f\t\t\t%0.2f\t%0.2f\t%0.2f",
	     curprim->id,
	     strlen(curprim->primname) > 0 ? curprim->primname : "None",
	     curprim->object,
	     ((Circle *)curprim)->centerx,
	     ((Circle *)curprim)->centery,
	     ((Circle *)curprim)->radius,
	     ((Circle *)curprim)->startarc,
	     ((Circle *)curprim)->endarc
	     );
    break;
  case PR_ELLIPSE:
    snprintf(prim_string, 1024, \
	     "%i\t%s\t%i\tEllipse\t%0.2f\t%0.2f\t\t\t%0.2f\t%0.2f\t%0.2f",
	     curprim->id,
	     strlen(curprim->primname) > 0 ? curprim->primname : "None",
	     curprim->object,
	     ((Ellipse *)curprim)->centerx,
	     ((Ellipse *)curprim)->centery,
	     ((Ellipse *)curprim)->semix,
	     ((Ellipse *)curprim)->startarc,
	     ((Ellipse *)curprim)->endarc
	     );
    break;
  case PR_DATA:
    snprintf(prim_string, 1024, \
	     "%i\t%s\t%i\tData\t%0.2f\t%0.2f\t\t\t%i\t%0.2f",
	     curprim->id,
	     strlen(curprim->primname) > 0 ? curprim->primname : "None",
	     curprim->object,
	     ((Data *)curprim)->trasx,
	     ((Data *)curprim)->trasy,
	     ((Data *)curprim)->npoints,
	     ((Data *)curprim)->rotation
	     );
    break;
  default:
    memset(prim_string, 0, MAX_PRIM_STRING);
    break;
  }
  return prim_string;
}

Primitive *u2d_Core::get_prim_by_label(const char *label) {
  int id = atoi(label);
  Primitive *curprim = firstprimitive;
  while(curprim){
    if(curprim->id == id){
      return curprim;
    }
    else {
      curprim = curprim->nextprimitive;
    }
  }
  fprintf(stderr, "Primitive not found! Browser list and primitive " \
	  "list missmatch! Or id %i unexistant!!\n", id);
  return NULL;
}

int u2d_Core::get_vertex_n(){
  return vertex_n;
}

void u2d_Core::find_objects() {
  if(firstprimitive == NULL) {
    /* Primitive list is still empty */
    return;
  }
  /* Here we reset all the variables used by calculation:
   * * object in each primitive
   * * object_part in each primitive (progressive object number)
   * * last_object global variable
   */
  unsigned int cur_element = 0;
  last_object = 0;
  bool attached = false;
  bool reverse = false;
  Primitive *curprim = firstprimitive;
  vertex_n = 0;
  while(curprim){
    curprim->element = 0;
    curprim->object = 0;
    curprim->object_part = 0;
    curprim->beginvertex = 0;
    curprim->endvertex = 0;
    curprim = curprim->nextprimitive;
  }


  /* THE algo */
  curprim = firstprimitive;
  Primitive *otherprim = firstprimitive;
  Primitive *firstprim = NULL;
  Primitive *lastprim = NULL;
  while(curprim){
    if(!curprim->object) {
      /* We do not flag the current primitive as used, so it stay in
       * the comparison pool of primitives. This way when we are the
       * last primitive and we are looking back for the first (the
       * one from witch the object start) we can find it among
       * those that are unflagged.
       */
      /* curprim->object = ++last_object; */
      last_object++;
      curprim->element = ++cur_element;
      curprim->object_part++;
      attached = false;
      reverse = false;
      otherprim = firstprimitive;
      firstprim = curprim;
      lastprim = curprim;
      while(otherprim){
	attached = false;
	reverse = false;
	/* Primitive is allready taken or is the curprim itself*/
	if(otherprim->object || (otherprim == curprim)) {
	  otherprim = otherprim->nextprimitive;
	}
	else {

	  if(curprim->beginvertex == 0 && \
	     isnear(DEFAULT_TOLL, curprim->x1, curprim->y1, otherprim->x1, otherprim->y1)){
	    vertex_n++;
	    curprim->beginvertex = vertex_n;
	    otherprim->beginvertex = vertex_n;
	    attached = true;
	    reverse = true;
	  }
	  else if(curprim->beginvertex == 0 && \
		  isnear(DEFAULT_TOLL, curprim->x1, curprim->y1, otherprim->x2, otherprim->y2)){
	    vertex_n++;
	    curprim->beginvertex = vertex_n;
	    otherprim->endvertex = vertex_n;
	    attached = true;
	  }

	  if(curprim->endvertex == 0 && \
	     isnear(DEFAULT_TOLL, curprim->x2, curprim->y2, otherprim->x1, otherprim->y1)){
	    vertex_n++;
	    curprim->endvertex = vertex_n;
	    otherprim->beginvertex = vertex_n;
	    attached = true;
	  }
	  else if(curprim->endvertex == 0 && \
		  isnear(DEFAULT_TOLL, curprim->x2, curprim->y2, otherprim->x2, otherprim->y2)){
	    vertex_n++;
	    curprim->endvertex = vertex_n;
	    otherprim->endvertex = vertex_n;
	    attached = true;
	    reverse = true;
	  }

	  if(reverse == true){
	    otherprim->reverse();
	  }

	  if(attached == true){
	    /* Primitive otherprim is attached */
	    otherprim->object = last_object;
	    /* We check that the current primitive is really the last
	     * before assigning it to lastprim, since maybe we are
	     * just getting the firstprim again from the pool
	     * to close the object. Remember from above note that
	     * we did not flagged the firstprimitive of the object
	     * as used.
	     *
	     * The line that increment the object part is a little
	     * hack. Because we want that the primitive that contain
	     * the first vertex (and maybe the last one) of it's
	     * closed object is numered as object_part = 1. It worked
	     * correctly outside this 'if' until when we begin to not
	     * flag the first primitive as taken.
	     * Now the primitive with vertex 1 can do this
	     * loop two time because we may get it from the primitive
	     * pool a second time while we look for the last vertex,
	     * in that case it would happen that the primitive
	     * object_part get renumered again. This fix this little
	     * problem by numbering only the primitives that are not
	     * the first.
	     */
	    if(otherprim != firstprim){
	      otherprim->element = ++cur_element;
	      otherprim->object_part = curprim->object_part + 1;
	      lastprim = otherprim;
	    }
	    curprim = otherprim;
	    otherprim = firstprimitive;
	  }
	  else {
	    /* Primitive otherprim is not attached */
	    otherprim = otherprim->nextprimitive;
	  }
	}


	/* firstprim and lastprim should be the first and
	 * last primitive of the object
	 */
	if(otherprim == NULL){
	  /* We passed the entire list without finding
	   * anymore primitives to add to this object
	   */
	  if(firstprim == lastprim){
#ifdef DEBUGGING
	    printf("Primitive %s is closed on itself\n", get_prim_string(firstprim));
#endif
	    /* Notice that in this case:
	     * firstprim == lastprim == curprim
	     */
	    firstprim->beginvertex = ++vertex_n;
	    firstprim->endvertex = vertex_n;
	    firstprim->object = last_object;
	  }
	}

      }
    }
    curprim = curprim->nextprimitive;
  }


  /* Sanity check */
  /* Check that each object is closed */

  /* Guesing of reference coorinates */

  double ref_x = 0;
  double ref_y = 0;

  Primitive *prim = firstprimitive;

  while(prim){
    switch(prim->type){
    case PR_LINE:
      ref_x = ((Line *)prim)->x1 > ref_x ? ((Line *)prim)->x1 : ref_x;
      ref_x = ((Line *)prim)->x2 > ref_x ? ((Line *)prim)->x2 : ref_x;
      ref_y = ((Line *)prim)->y1 > ref_y ? ((Line *)prim)->y1 : ref_y;
      ref_y = ((Line *)prim)->y2 > ref_y ? ((Line *)prim)->y2 : ref_y;
      break;

    case PR_CIRCLE:
      ref_x = (((Circle *)prim)->centerx + ((Circle *)prim)->radius) > ref_x ? \
	(((Circle *)prim)->centerx + ((Circle *)prim)->radius) : ref_x;
      ref_y = (((Circle *)prim)->centery + ((Circle *)prim)->radius) > ref_y ? \
	(((Circle *)prim)->centery + ((Circle *)prim)->radius) : ref_y;
      break;

    case PR_ELLIPSE:
      ref_x = (((Ellipse *)prim)->centerx + ((Ellipse *)prim)->semix) > ref_x ? \
	(((Ellipse *)prim)->centerx + ((Ellipse *)prim)->semix) : ref_x;
      ref_y = (((Ellipse *)prim)->centery + ((Ellipse *)prim)->semiy) > ref_y ? \
	(((Ellipse *)prim)->centery + ((Ellipse *)prim)->semiy) : ref_y;
      break;

    case PR_DATA:
      for(int i = 1; i < (((Data *)prim)->npoints - 1)*2; i = i + 2) {
	ref_x = rot_matrixX(((Data *)prim)->rotation,\
			   ((Data *)prim)->points[i - 1] + ((Data *)prim)->trasx,\
			   ((Data *)prim)->points[i] + ((Data *)prim)->trasy)\
	  > ref_x ?\
	  rot_matrixX(((Data *)prim)->rotation,\
		      ((Data *)prim)->points[i - 1] + ((Data *)prim)->trasx,\
		      ((Data *)prim)->points[i] + ((Data *)prim)->trasy)\
	  : ref_x;
	ref_y = rot_matrixY(((Data *)prim)->rotation,\
			   ((Data *)prim)->points[i - 1] + ((Data *)prim)->trasx,\
			   ((Data *)prim)->points[i] + ((Data *)prim)->trasy)\
	  > ref_y ?\
	  rot_matrixX(((Data *)prim)->rotation,\
		      ((Data *)prim)->points[i - 1] + ((Data *)prim)->trasx,\
		      ((Data *)prim)->points[i] + ((Data *)prim)->trasy)\
	  : ref_y;
      }
      break;
    }
    prim = prim->nextprimitive;
  }
  /* We take as reference a point that is 0.1 times 
   * farter than current reference. */
  ref_x = ref_x + 0.1 * ref_x;
  ref_y = ref_y + 0.1 * ref_y;

  /* External boundary search */
  /* The object with bigger area is the one that is external */
  unsigned int biggest_object = 0;
  double biggest_object_area = 0;

  for(unsigned int cur_obj = 1; cur_obj <= last_object; cur_obj++){
    double area = 0;
    curprim = firstprimitive;
    while(curprim){
      if(curprim->object == cur_obj){
	area = area + curprim->calc_area(ref_x, ref_y);
      }
      curprim = curprim->nextprimitive;
    }
    if(fabs(area) > fabs(biggest_object_area)){
      biggest_object_area = area;
      biggest_object = cur_obj;
    }
    else if(fabs(area) == fabs(biggest_object_area)){
      /* Minor consistency check */
      fprintf(stderr, "Cannot guess correct bounday! "\
	      "Two objects have the same area\n");
    }
  }

  /* We have discovered which is the external boundary,
   * now we flag it as boundary */
  curprim = firstprimitive;
  while(curprim){
    if(curprim->object == biggest_object){
      curprim->isboundary = true;
    }
    curprim = curprim->nextprimitive;
  }


  /* Checking clockwiseness */
  /* Algo:
   * Start with first object number,
   * Calculate area of each element of that object and sum it
   * If area is positive (numeration is clockwise) and the object is
   *   not boundary it is ok, elsewhere reverse numeration.
   *   if negative (numeration is counter-clockwise) and the object is
   *   boundary it is ok, elsewhere reverse numeration.
   * Next object
   * Finished object, exit
   */
  for(unsigned int cur_obj = 1; cur_obj <= last_object; cur_obj++){
    double area = 0;
    bool boundary = false;
    int last_vertex = 0;
    int first_vertex = 0;
    curprim = firstprimitive;
    while(curprim){
      if(curprim->object == cur_obj){
	area = area + curprim->calc_area(ref_x, ref_y);
	boundary = curprim->isboundary; /* Implies above sanity check */
      }
      curprim = curprim->nextprimitive;
    }
    if(((area > 0) && (boundary == true)) || ((area < 0) && (boundary == false))){
      /* Reverse the object numeration */
      printf("\nReversing numeration of object %i, area: %f\n", cur_obj, area);
      curprim = firstprimitive;
      while(curprim){
	if(curprim->object == cur_obj){
	  curprim->beginvertex > last_vertex ? last_vertex = curprim->beginvertex : NULL;
	  curprim->endvertex > last_vertex ? last_vertex = curprim->endvertex : NULL;
	  if(first_vertex == 0){
	    first_vertex = curprim->beginvertex;
	  }
	  curprim->beginvertex < first_vertex ? first_vertex = curprim->beginvertex : NULL;
	  curprim->endvertex < first_vertex ? first_vertex = curprim->endvertex : NULL;
	}
	curprim = curprim->nextprimitive;
      }
      curprim = firstprimitive;
      while(curprim){
	if(curprim->object == cur_obj){
	  curprim->beginvertex = last_vertex - curprim->beginvertex + first_vertex;
	  curprim->endvertex = last_vertex - curprim->endvertex + first_vertex;
	  curprim->reverse();
	}
	curprim = curprim->nextprimitive;
      }
    }
#ifdef DEBUGGING
    printf("\nObject %i area : %f\n", cur_obj, area);
#endif
  }

  /* Sort the list */
  /* Why we sort the list?
   * We do that because when we output the list we want it
   * to be sorted with ->element as sorting index. This is
   * because umesh2d wants the elements in order.
   * After some thinking I come out with the idea that sorting
   * and then outputting the already ordered list is
   * faster than searching for each element in order and then
   * and outputting it.
   * This result come from the fact that
   * optimizations tend to work better on repetitive works
   * so doing all of one thing and then all of the other will
   * probably give better results because the compiler can
   * better undestand what we want to do and optimize it more
   * than a mixed way.
   */
  /* Which sorting algo?
   * Quicksort is nice we all know, but it's complex and it's for
   * BIG database.
   *
   * Using qsort() is nice idea... but it's too easy :) and
   * require I arrange the list in an array and then regenerate
   * the list, maybe it can be a bit slow cause of that.
   *
   * Heapsort can be a candidate, it's not too complex and it's
   * fast as quicksort in many cases. Saddly it's very unadapt
   * to our case.
   *
   * Bubble sort, no way, you know :)
   *
   * Insertion sort, is very good for this, since it's very fast
   * for small databases and very fast for substantially ordered
   * databases, it's so fast that highly optimized quicksort use
   * it to sort the database when they fragment them enought.
   * We have both, user will normally put elements ordered and
   * element number is normally limited.
   * Moreover insertion with linked list has the lesser
   * overhead.
   *
   * Gabry Sort is something my brain just put out, it's
   * basically a sort that exploits the fact that I know
   * each element appears only 1 time and that I know
   * in advance which is the last element. This way I can
   * just use a smart insertion sort to create a new list
   * that is ordered.
   * The next lines have no sense ^_^
   * Take element 1 from list A put in B
   * element 2 then connect to list B, and so on...
   * It's a dumb sort!
   *
   * Counting sort look very suited as it is Radix sort since
   * they both perform the same way in this situation
   *
   * Merge Sort is the suggested sort for linked lists
   * because it can be coded without the memory overhead
   * and is as fast as Quicksort
   */

  /*
 void insertSort(int a[], size_t length) {
     int i, j;
 
     for(i = 1; i < length; i++) {
         int value = a[i];
         for (j = i - 1; j >= 0 && a[j] > value; j--) {
             a[j + 1] = a[j];
         }
         a[j + 1] = value;
     }
 }
  */

  /*
    void Sort(float array[], int begin, int end) 
    {
    int mid;
    if (end - begin <= 1) return;
    mid = (begin + end) / 2;
    Sort(array, begin, mid);
    Sort(array, mid, end);
    Merge(array, begin, mid, end);
    }
  */


  /* This is insertion sort, can be improved. */
  curprim = firstprimitive->nextprimitive;
  while(curprim != NULL){
    if(curprim->element > curprim->prevprimitive->element) {
      /* This part of the list should be already ordered */
    }
    else {
      /* We found that curprim->nextptimitive is not ordered,
       * it is greater than the the curprim
       * we move it to correct position */

      /* We handle the case where curprim->nextprimitive should be the first element */
      if(curprim->element < firstprimitive->element) {
	/* curprim should be the first element */
	remove_prim(curprim);
	curprim->nextprimitive = firstprimitive;
	firstprimitive->prevprimitive = curprim;
	firstprimitive = curprim;
      }
      else {  
	/* Search for the right location and insert the primitive*/

	/* This is a temporary hack.. since I need this working
	 * and have a few time.
	 * Remove the latter if..else and  then change
	 * move_insert_down_prim()
	 * with 
	 * move_insert_up_prim()
	 */
	if(firstprimitive->nextprimitive != NULL){
	  otherprim = firstprimitive->nextprimitive;
	}
	else {
	  /* In this case the while() will exit immedialty */
	  otherprim = firstprimitive;
	}
	while(otherprim->nextprimitive != NULL && (curprim->element > otherprim->element)) {
	  otherprim = otherprim->nextprimitive;
	}
	move_insert_down_prim(otherprim->prevprimitive, curprim);
      }
    }
    curprim = curprim->nextprimitive;
  }
}

 
int u2d_Core::allocate_steiner(int number) {
  if(steiners != NULL) {
    free(steiners);
  }
  steiners = (steiner *)malloc(number * sizeof(steiner));
  if(steiners == NULL){
    steiner_max_n = 0;
    return NOMEM;
  }
  steiner_n = 0;
  steiner_max_n = number;
  return SUCESS;
}
 
int u2d_Core::allocate_backgrid(int number) {
  if(backgrids != NULL) {
    free(backgrids);
  }
  backgrids = (backgrid *)malloc(number * sizeof(backgrid));
  if(backgrids == NULL){
    backgrid_max_n = 0;
    return NOMEM;
  }
  backgrid_n = 0;
  backgrid_max_n = number;
  return SUCESS;
  
}

void u2d_Core::set_steiner(int position, double x, double y) {
  steiners[position].x = x;
  steiners[position].y = y;
  steiner_n++;
}

void u2d_Core::set_backgrid(int position, double x, double y, double hdata) {
  backgrids[position].x = x;
  backgrids[position].y = y;
  backgrids[position].hdata = hdata;
  backgrid_n++;
}

void u2d_Core::debug_cb(){
  Primitive *curprim = firstprimitive;
  while(curprim){
    printf("op:%i bv:%i ev:%i prim:%s\n",\
	   curprim->object_part,\
	   curprim->beginvertex,\
	   curprim->endvertex,\
	   get_prim_string(curprim));
    printf("calling calc_area() : %f\n", curprim->calc_area(0, 0));
    curprim = curprim->nextprimitive;
  }
}

