MODULE boundary_2d

  USE curves          !  Curves module

  USE grid_types      !  Types module

  USE list            !  List module

  USE delaunay        !  Delaunay basic module

  USE eucl_delaunay   !  Delaunay Euclid module

  USE metric_2d       !  metric module


  IMPLICIT NONE

  PUBLIC :: boundary_grid, add_points_1d, add_points_12d, stitch_boundary, &
       & halve_bou

CONTAINS
!  
  SUBROUTINE boundary_grid (grid)

    TYPE (domain), INTENT(INOUT) :: grid

    CALL bou_r_points

    CALL set_grid
    CALL add_points_12d (grid)

  CONTAINS

    !
    
    SUBROUTINE bou_r_points

      INTEGER :: i, j, n, ns
      INTEGER :: ii, i_l, i_r
      TYPE (point), POINTER :: h1, p1, p2
      REAL (KIND=8), DIMENSION(:), POINTER :: s, h, sdata
      
      NULLIFY ( s, h)

      !  1d points
      write(*,*)'Warning: n majored to obtain l<1'
      DO i=1,grid % number_of_edges
         h1  =>  grid % edges(i) % add_head

         !final number of points determining(metric imposal)
         CALL metric_r_1d (i, s, h)
         CALL points_1d (h1, s, h)

         p1  =>  h1
         n = 0
         DO; p1  =>  p1 % next; IF ( ASSOCIATED (p1, h1) ) EXIT
            n = n+1
            p1 % cnd = i
            p1 % old  =>  p1 % prev
         ENDDO

         h1 % next % cnd = -i
         h1 % prev % cnd = -i
         NULLIFY (h1 % next % old)

         WRITE (*,'(a9,i2,a11,i4)') ' Segment ',i,'; points = ',n
      ENDDO

      !  2d points: vertex points

      DO i=1,grid % number_of_verts
         p2  =>  new_point()
         grid % verts(i) % pnt  =>  p2
         CALL pushtop_point (grid % add_head, p2)
      ENDDO

      !  2d points: edge points
      ! completes a list of points and estabilishes a 1d->2d connection
      ! between boundary and domain points  
      
      DO i=1,grid % number_of_edges
         ns = SIZE(grid%edges(i)%idata)
         sdata => grid%edges(i)%sdata         
         j = grid % edges(i) % index

         h1  =>  grid % edges(i) % add_head !Riemann points edge head
         p1  =>  h1 % next

         ! beginning edge point
         p2  =>  grid % verts(grid % edges(i) % begin_vert) % pnt !(*)
                                              !vertex associated points
         p1 % ppp  =>  p2                     !p1 is the list's first element
         
         p2 % cnd = p1 % cnd                  ! belonging to edge
         CALL evalc_0 (j, p1 % x, p2 % x)

         i_l = 1	! index left    
         i_r = 1	! index right
        
         CALL attr_metr(grid, j, p1, i_l, i_r, p2)

         h1  =>  h1 % prev                                     
         DO; p1  =>  p1 % next; IF ( ASSOCIATED (p1, h1) ) EXIT
            
            ! 1d -> 2d correspondence
            p2  =>  new_point()
            p1 % ppp  =>  p2    
            p2 % cnd = p1 % cnd 

            CALL evalc_0 (j, p1 % x, p2 % x)

            !  metric of point

            DO ii = ns, i_l, -1
               IF((sdata(ii)-p1%x(1)) <= 0.d0) THEN
                  i_l = ii
                  EXIT
               ENDIF
            ENDDO
            DO ii = i_l, ns
               IF((sdata(ii)-p1%x(1)) >= 0.d0) THEN
                  i_r = ii
                  EXIT
               ENDIF
            ENDDO

            CALL attr_metr(grid, j, p1, i_l, i_r, p2) 
            CALL pushtop_point (grid % add_head, p2)  
         ENDDO

         !final edge point
         p2  =>  grid % verts(grid % edges(i) % end_vert) % pnt
         p1 % ppp  =>  p2
         p2 % cnd = p1 % cnd
         !no pushtop needed;enough to insert the corresponding vertex
         !by DD
         IF(grid%edges(i)%begin_vert /= grid%edges(i)%end_vert) THEN
         
            CALL evalc_0 (j, p1 % x, p2 % x)
            CALL attr_metr(grid, j, p1, ns, ns, p2)
         
         ENDIF

      ENDDO
      
    END SUBROUTINE bou_r_points
    
    !
    
    SUBROUTINE metric_r_1d (c, s, h)

      INTEGER, INTENT(IN) :: c
      REAL (KIND=8), DIMENSION(:), POINTER :: s, h

      INTEGER :: ns, ms, i, j, j0, jm, edge_kind
      REAL (KIND=8) :: ss, s0, h0, si, hi, fs, fh, sh, dh, tj
      INTEGER,       DIMENSION(:)    , POINTER :: idata
      REAL (KIND=8), DIMENSION(:)    , POINTER :: sdata
      REAL (KIND=8), DIMENSION(:,:)  , POINTER :: hdata

      CALL evalc_s (c, ns, s)

      IF ( ASSOCIATED ( h ) ) DEALLOCATE ( h )
      ALLOCATE ( h(ns) )

      edge_kind = grid % edges(c) % edge_kind

      idata  =>  grid % edges(c) % idata
      sdata  =>  grid % edges(c) % sdata
      hdata  =>  grid % edges(c) % hdata

      ms = SIZE(idata)

      !  Compute h(s) --- interpolated local spacing at curve knots

      j0 = 1
      ss = s(ns)
      s0 = ss*sdata(1)

      !  First point projected metric

      call proj_metr(c, edge_kind, sdata(1), hdata(:,1), h0)

      h(j0) = h0

      DO i=2,ms

         jm = j0
         si = ss*sdata(i)
         call proj_metr(c, edge_kind, sdata(i), hdata(:,i), hi)

         DO j=j0+1,ns
            IF ( s(j) >= si ) THEN
               jm = j-1
               EXIT
            ENDIF
         ENDDO

         IF ( jm == j0 ) THEN
            WRITE (*,*) 'build_h error for si = ',si
            WRITE (*,*) '--STOP-- (metric_r_1d)'
            STOP
         ENDIF

         SELECT CASE (idata(i))

         CASE (1) ! Linear interpolation

            fs = 1.0/(si-s0)
            dh = hi-h0
            DO j=j0+1,jm
               tj   = fs*(s(j)-s0)
               h(j) = h0+tj*dh
            ENDDO

         CASE (2) ! Geometrical interpolation

            fs = 1.0/(si-s0)
            fh = hi/h0
            DO j=j0+1,jm
               tj   = fs*(s(j)-s0)
               h(j) = h0*(fh**tj)
            ENDDO

         CASE (3) ! Sinusoidal interpolation

            fs = pi/(si-s0)
            sh = 0.5*(hi+h0); dh = 0.5*(hi-h0)
            DO j=j0+1,jm
               tj   = fs*(s(j)-s0)
               h(j) = sh-dh*cos(tj)
            ENDDO

         END SELECT

         j0 = jm; s0 = si; h0 = hi

      ENDDO

      CALL proj_metr(c, edge_kind, sdata(ms), hdata(:,ms), h(ns))

      !  Integral of 1/h(s) --- riemannian curvilinear abscissa at knots

      hi   = 1.0/h(1)
      h(1) = 0.0
      DO i=2,ns
         h0   = hi
         hi   = 1.0/h(i)
         dh   = 0.5*(s(i)-s(i-1))*(hi+h0)
         !**         if(dh > .5d0) then
         !**            write(*,*)'need a point! dh > 1/2 (metric_r_1d)'
         !**         endif
         h(i) = h(i-1) + dh
      ENDDO

    END SUBROUTINE metric_r_1d
    !
    !********************************************************************
    !
    SUBROUTINE points_1d (head, s, h)

      TYPE(point)               , POINTER :: head
      REAL(KIND=8), DIMENSION(:), POINTER :: s, h


      INTEGER :: ns, n, j0, i, j
      REAL (KIND=8) :: hh, ss, dh, hi, si
      TYPE (point), POINTER :: p


      !  Subdivide the curve into L segments of unit riemannian
      !  length, L being the riemannian length of the curve.

      ns = SIZE(s)
      hh = h(ns)
      n  = MAX(INT(hh),1)
      !write(*,*)'Warning: n majored to obtain l<1'
      !**      IF ( n*(n+1) < hh**2 ) n = n+1
      !**      write(*,*)'n', n
      dh = hh/float(n)
      if(dh > 1.d0) then
         n = n+1
         dh = hh/float(n)
      endif
      ss = 1.0/s(ns)

      j0 = 1
      p  =>  new_point ()
      p % x(1) = 0.0

      CALL pushbot_point (head, p)

      DO i=2,n
         hi = dh*(i-1)
         DO j=j0+1,ns
            IF ( h(j) >= hi ) THEN
               j0 = j-1
               si = s(j-1) + (hi-h(j-1))*(s(j)-s(j-1))/(h(j)-h(j-1))

               p  =>  new_point ()
               p % x(1) = si*ss   !'Riemann abscissa'

               CALL pushbot_point (head, p)
               EXIT
            ENDIF
         ENDDO
      ENDDO

      p  =>  new_point ()
      p % x(1) = 1.0

      CALL pushbot_point (head, p)

    END SUBROUTINE points_1d
    !
    !********************************************************************
    !
    SUBROUTINE set_grid

      INTEGER :: i
      REAL (KIND=8), DIMENSION(:), ALLOCATABLE :: xlb, xrt
      TYPE (point), POINTER :: hap, p
      TYPE (simplex), POINTER :: has, hs
  
      !  1d grids

      ALLOCATE ( xlb(nd_b) )
      ALLOCATE ( xrt(nd_b) )

      xlb = 0.
      xrt = 1.
      
      DO i=1,grid % number_of_edges
         
         ! 1d box definition
         hap  =>  grid % edges(i) % axp_head  ! auxiliary point head
         has  =>  grid % edges(i) % axs_head  ! auxiliary simplex head
         hs   =>  grid % edges(i) % spx_head  ! simplex head
         CALL set_delaunay (nd_b, xlb, xrt, hap, has, hs)
      
      ENDDO

      DEALLOCATE ( xlb )
      DEALLOCATE ( xrt )


      !  2d grid

      ALLOCATE ( xlb(nd_d) )
      ALLOCATE ( xrt(nd_d) )
      
      xlb = +HUGE(xlb)
      xrt = -HUGE(xrt)

      !  find max and min  coordinates values
      !  (box must include every point)
      p  =>  grid % add_head      
      DO; p  =>  p % next; IF ( ASSOCIATED (p, grid % add_head) ) EXIT
         xlb = MIN(xlb, p % x); xrt = MAX(xrt, p % x)
      END DO
      
      ! 2d box definition
      hap  =>  grid % axp_head
      has  =>  grid % axs_head
      hs   =>  grid % spx_head

      CALL set_delaunay (nd_d, xlb, xrt, hap, has, hs)

      DEALLOCATE ( xlb )
      DEALLOCATE ( xrt )

    END SUBROUTINE set_grid

  END SUBROUTINE boundary_grid
  
  !
  
  SUBROUTINE add_points_1d (grid)

    TYPE (domain), INTENT(INOUT) :: grid

    INTEGER :: i
    TYPE (point), POINTER :: p, q
    TYPE (simplex), POINTER :: hs, hi, he
    DO i=1,grid % number_of_edges

       hs  =>  grid % edges(i) % spx_head
       hi  =>  grid % edges(i) % int_head
       he  =>  grid % edges(i) % ext_head

       CALL merge_ie (hs, hi, he)
       ! 
       ! int_head and ext_head pushtop in spx_head
       ! put status=und to every simplex
       
       p  =>  grid % edges(i) % add_head % next
       
       DO; IF ( ASSOCIATED (p, grid % edges(i) % add_head) ) EXIT
          q  =>  p % next;
  
          CALL insert_point  (nd_b, hs, p)
          CALL movetop_point (grid % edges(i) % pnt_head, p)
               !inserted points list
          p  =>  q
       END DO
       
   
       
       CALL split_ie_1d  (nd_b, hs, hi, he)
       ! moves in int the simplices having status /= frz
       !          ext the simplices having status = frz
       
    ENDDO

  END SUBROUTINE add_points_1d
  
  !
  
  SUBROUTINE add_points_12d (grid)

    TYPE (domain), INTENT(INOUT) :: grid

    INTEGER :: i, nstc
    TYPE (point), POINTER :: p, q
    TYPE (simplex), POINTER :: hs, hi, he, hbi, hbs

    integer :: k

    !  1d points

    CALL add_points_1d (grid)

    !  2d points

    hs  =>  grid % spx_head
    hi  =>  grid % int_head
    he  =>  grid % ext_head

    k = 0

    add_2d_points: DO

       k = k+1

       CALL merge_ie (hs, hi, he)

       p  =>  grid % add_head % next

       DO; IF ( ASSOCIATED (p, grid % add_head) ) EXIT
          q  =>  p % next

          CALL insert_point  (nd_d, grid % spx_head, p)
          CALL movetop_point (grid % pnt_head, p)
               !inserted points list  
          p  =>  q
       END DO


       DO i=1,grid % number_of_edges
          hbi  =>  grid % edges(i) % int_head
          hbs  =>  grid % edges(i) % stc_head
          CALL flag_shell (nd_d, hbi, hs, hbs)
          
       ENDDO

       CALL stitch_boundary (grid, nstc)

       IF ( nstc .GT. 0 ) THEN

          WRITE (*,*) 'Stitch points   = ',nstc          
          CALL add_points_1d (grid)

       ELSE
          ! gets from spx_h internal and external boundary adjacent simplices 
          ! then spreads the internal and external status
          ! except for frozen status simplices(box adjacents)
          CALL split_ie (nd_d, hs, hi, he)
          EXIT

       ENDIF

    ENDDO add_2d_points

  END SUBROUTINE add_points_12d
  
  !
  
  SUBROUTINE stitch_boundary (grid, nadd)

    TYPE (domain), INTENT(INOUT) :: grid
    INTEGER, INTENT(OUT) :: nadd

    INTEGER :: e, g, n
    TYPE (simplex), POINTER :: head, s

    n = 0

    DO e=1,grid % number_of_edges
       g = grid % edges(e) % index; head  =>  grid % edges(e) % stc_head

       s  =>  head
       DO
          s  =>  s % next
          IF ( ASSOCIATED (s, head) ) EXIT

          !**** TEST
          if(ABS(s % opp(1) % pnt %cnd) /= ABS(s % opp(2) % pnt %cnd)) then
             write(*,*)'edge: first', s % opp(1) % pnt %cnd
             write(*,*)'edge:second', s % opp(2) % pnt %cnd
          endif
          !**** TEST

          n = n+1
          CALL halve_bou (grid, e, g, s)
       ENDDO

    ENDDO

    nadd = n


  END SUBROUTINE stitch_boundary
  
  !
  
  SUBROUTINE halve_bou (grid, ibou, igeo, sbou)

    TYPE (domain), INTENT(INOUT) :: grid
    TYPE (simplex), TARGET, INTENT(IN) :: sbou
    INTEGER :: ibou, igeo

    INTEGER :: i
    REAL (KIND=8) :: factor
    TYPE (point), POINTER :: p_dom, p_bou

    factor = 1./DBLE(nd_b+1)

    p_bou  =>  new_point()
    p_dom  =>  new_point()

    p_bou % cnd = ibou
    IF(ASSOCIATED(sbou % int)) THEN		! by trive
       p_bou % parent = sbou % int % index
    ELSE
       p_bou % parent = 0
    ENDIF
    p_bou % old  =>  sbou % opp(1) % pnt
    p_bou % ppp  =>  p_dom
    p_bou % x = 0.
    DO i=1,nd_b+1
       p_bou % x = p_bou % x + sbou % opp(i) % pnt % x
    ENDDO
    p_bou % x = factor * p_bou % x

    p_dom % cnd = ibou
    IF(ASSOCIATED(sbou % int)) THEN		! by trive
       p_dom % parent = sbou % int % index
    ELSE
       p_dom % parent = 0
    ENDIF
    p_dom % old  =>  sbou % opp(1) % pnt % ppp

    p_dom % metric = 0.
    DO i=1,nd_b+1
       p_dom % metric = p_dom % metric + sbou % opp(i) % pnt % ppp % metric
    ENDDO
    p_dom % metric = factor * p_dom % metric

    CALL evalc_0       (igeo, p_bou % x, p_dom % x)
    CALL pushtop_point (grid % edges(ibou) % add_head, p_bou)
    CALL pushtop_point (grid % add_head, p_dom)

  END SUBROUTINE halve_bou
  
  !
  
END MODULE boundary_2d
